@@ -163,8 +163,6 @@ njs_array_convert_to_slow_array(njs_vm_t *vm, njs_array_t *array)
         }
     }
 
-    /* GC: release value. */
-
     njs_mp_free(vm->mem_pool, array->data);
     array->start = NULL;
 
@@ -338,7 +336,6 @@ njs_array_add(njs_vm_t *vm, njs_array_t *array, njs_value_t *value)
     ret = njs_array_expand(vm, array, 0, 1);
 
     if (njs_fast_path(ret == NJS_OK)) {
-        /* GC: retain value. */
         array->start[array->length++] = *value;
     }
 
@@ -466,7 +463,6 @@ njs_array_constructor(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
             } else {
                 while (size != 0) {
-                    njs_retain(args);
                     *value++ = *args++;
                     size--;
                 }
@@ -595,14 +591,14 @@ njs_array_of(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
         return NJS_ERROR;
     }
 
-    njs_set_array(retval, array);
-
     if (array->object.fast_array) {
         for (i = 0; i < length; i++) {
             array->start[i] = args[i + 1];
         }
     }
 
+    njs_set_array(retval, array);
+
     return NJS_OK;
 }
 
@@ -830,29 +826,15 @@ njs_array_prototype_slice_copy(njs_vm_t *vm, njs_value_t *this,
             src = string.start;
             end = src + string.size;
 
-            if (string.length == 0) {
-                /* Byte string. */
-                do {
-                    value = &array->start[n++];
-                    dst = njs_string_short_start(value);
-                    *dst = *src++;
-                    njs_string_short_set(value, 1, 0);
+            do {
+                value = &array->start[n++];
+                dst = njs_string_short_start(value);
+                dst = njs_utf8_copy(dst, &src, end);
+                size = dst - njs_string_short_start(value);
+                njs_string_short_set(value, size, 1);
 
-                    length--;
-                } while (length != 0);
-
-            } else {
-                /* UTF-8 or ASCII string. */
-                do {
-                    value = &array->start[n++];
-                    dst = njs_string_short_start(value);
-                    dst = njs_utf8_copy(dst, &src, end);
-                    size = dst - njs_string_short_start(value);
-                    njs_string_short_set(value, size, 1);
-
-                    length--;
-                } while (length != 0);
-            }
+                length--;
+            } while (length != 0);
 
         } else if (njs_is_object(this)) {
 
@@ -967,7 +949,6 @@ njs_array_prototype_push(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
             }
 
             for (i = 1; i < nargs; i++) {
-                /* GC: njs_retain(&args[i]); */
                 array->start[array->length++] = args[i];
             }
         }
@@ -1094,7 +1075,6 @@ njs_array_prototype_unshift(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
             do {
                 n--;
-                /* GC: njs_retain(&args[n]); */
                 array->start--;
                 array->start[0] = args[n];
             } while (n > 1);
@@ -1408,7 +1388,7 @@ njs_array_prototype_to_spliced(njs_vm_t *vm, njs_value_t *args,
 {
     int64_t      i, n, r, start, length, to_insert, to_skip, new_length;
     njs_int_t    ret;
-    njs_value_t  *this, value;
+    njs_value_t  *this, a, value;
     njs_array_t  *array;
 
     this = njs_argument(args, 0);
@@ -1459,7 +1439,7 @@ njs_array_prototype_to_spliced(njs_vm_t *vm, njs_value_t *args,
         return NJS_ERROR;
     }
 
-    njs_set_array(retval, array);
+    njs_set_array(&a, array);
 
     for (i = 0; i < start; i++) {
         ret = njs_value_property_i64(vm, this, i, &value);
@@ -1467,14 +1447,14 @@ njs_array_prototype_to_spliced(njs_vm_t *vm, njs_value_t *args,
             return NJS_ERROR;
         }
 
-        ret = njs_value_create_data_prop_i64(vm, retval, i, &value, 0);
+        ret = njs_value_create_data_prop_i64(vm, &a, i, &value, 0);
         if (njs_slow_path(ret != NJS_OK)) {
             return ret;
         }
     }
 
     for (n = 3; to_insert-- > 0; i++, n++) {
-        ret = njs_value_create_data_prop_i64(vm, retval, i, &args[n], 0);
+        ret = njs_value_create_data_prop_i64(vm, &a, i, &args[n], 0);
         if (njs_slow_path(ret != NJS_OK)) {
             return ret;
         }
@@ -1488,7 +1468,7 @@ njs_array_prototype_to_spliced(njs_vm_t *vm, njs_value_t *args,
             return NJS_ERROR;
         }
 
-        ret = njs_value_create_data_prop_i64(vm, retval, i, &value, 0);
+        ret = njs_value_create_data_prop_i64(vm, &a, i, &value, 0);
         if (njs_slow_path(ret != NJS_OK)) {
             return ret;
         }
@@ -1497,6 +1477,8 @@ njs_array_prototype_to_spliced(njs_vm_t *vm, njs_value_t *args,
         i++;
     }
 
+    njs_set_array(retval, array);
+
     return NJS_OK;
 }
 
@@ -1582,7 +1564,7 @@ njs_array_prototype_to_reversed(njs_vm_t *vm, njs_value_t *args,
     int64_t      length, i;
     njs_int_t    ret;
     njs_array_t  *array;
-    njs_value_t  *this, value;
+    njs_value_t  *this, a, value;
 
     this = njs_argument(args, 0);
 
@@ -1601,7 +1583,7 @@ njs_array_prototype_to_reversed(njs_vm_t *vm, njs_value_t *args,
         return NJS_ERROR;
     }
 
-    njs_set_array(retval, array);
+    njs_set_array(&a, array);
 
     for (i = 0; i < length; i++) {
         ret = njs_value_property_i64(vm, this, length - i - 1, &value);
@@ -1609,12 +1591,14 @@ njs_array_prototype_to_reversed(njs_vm_t *vm, njs_value_t *args,
             return NJS_ERROR;
         }
 
-        ret = njs_value_create_data_prop_i64(vm, retval, i, &value, 0);
+        ret = njs_value_create_data_prop_i64(vm, &a, i, &value, 0);
         if (njs_slow_path(ret != NJS_OK)) {
             return ret;
         }
     }
 
+    njs_set_array(retval, array);
+
     return NJS_OK;
 }
 
@@ -1653,11 +1637,10 @@ static njs_int_t
 njs_array_prototype_join(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     njs_index_t unused, njs_value_t *retval)
 {
-    u_char             *p, *last;
+    u_char             *p;
     int64_t            i, size, len, length;
     njs_int_t          ret;
     njs_chb_t          chain;
-    njs_utf8_t         utf8;
     njs_value_t        *value, *this, entry;
     njs_string_prop_t  separator, string;
 
@@ -1690,7 +1673,6 @@ njs_array_prototype_join(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     }
 
     length = 0;
-    utf8 = njs_is_byte_string(&separator) ? NJS_STRING_BYTE : NJS_STRING_UTF8;
 
     ret = njs_object_length(vm, this, &len);
     if (njs_slow_path(ret == NJS_ERROR)) {
@@ -1704,7 +1686,7 @@ njs_array_prototype_join(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     value = &entry;
 
-    njs_chb_init(&chain, vm->mem_pool);
+    NJS_CHB_MP_INIT(&chain, vm);
 
     for (i = 0; i < len; i++) {
         ret = njs_value_property_i64(vm, this, i, value);
@@ -1714,29 +1696,15 @@ njs_array_prototype_join(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
         if (!njs_is_null_or_undefined(value)) {
             if (!njs_is_string(value)) {
-                last = njs_chb_current(&chain);
-
                 ret = njs_value_to_chain(vm, &chain, value);
                 if (njs_slow_path(ret < NJS_OK)) {
                     return ret;
                 }
 
-                if (last != njs_chb_current(&chain) && ret == 0) {
-                    /*
-                     * Appended values was a byte string.
-                     */
-                    utf8 = NJS_STRING_BYTE;
-                }
-
                 length += ret;
 
             } else {
                 (void) njs_string_prop(&string, value);
-
-                if (njs_is_byte_string(&string)) {
-                    utf8 = NJS_STRING_BYTE;
-                }
-
                 length += string.length;
                 njs_chb_append(&chain, string.start, string.size);
             }
@@ -1761,7 +1729,7 @@ njs_array_prototype_join(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     length -= separator.length;
 
-    p = njs_string_alloc(vm, retval, size, utf8 ? length : 0);
+    p = njs_string_alloc(vm, retval, size, length);
     if (njs_slow_path(p == NULL)) {
         return NJS_ERROR;
     }
@@ -1874,8 +1842,8 @@ njs_array_keys(njs_vm_t *vm, njs_value_t *object, njs_bool_t all)
 {
     njs_array_t  *keys;
 
-    keys = njs_value_own_enumerate(vm, object, NJS_ENUM_KEYS, NJS_ENUM_STRING,
-                                   all);
+    keys = njs_value_own_enumerate(vm, object, NJS_ENUM_KEYS | NJS_ENUM_STRING
+                                   | (!all ? NJS_ENUM_ENUMERABLE_ONLY : 0));
     if (njs_slow_path(keys == NULL)) {
         return NULL;
     }
@@ -2133,8 +2101,6 @@ njs_array_iterator_call(njs_vm_t *vm, njs_iterator_args_t *args,
 {
     njs_value_t  arguments[3];
 
-    /* GC: array elt, array */
-
     arguments[0] = *entry;
     njs_set_number(&arguments[1], n);
     njs_value_assign(&arguments[2], &args->value);
@@ -2303,8 +2269,6 @@ njs_array_handler_reduce(njs_vm_t *vm, njs_iterator_args_t *args,
             return NJS_OK;
         }
 
-        /* GC: array elt, array */
-
         njs_set_undefined(&arguments[0]);
         njs_value_assign(&arguments[1], &args->argument);
         arguments[2] = *entry;
@@ -3058,7 +3022,7 @@ njs_array_prototype_to_sorted(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     int64_t                i, nslots, nunds, length;
     njs_int_t              ret;
     njs_array_t            *array;
-    njs_value_t            *this, *comparefn;
+    njs_value_t            *this, *comparefn, a;
     njs_function_t         *compare;
     njs_array_sort_slot_t  *slots;
 
@@ -3110,24 +3074,25 @@ njs_array_prototype_to_sorted(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
 
     njs_assert(length == (nslots + nunds));
 
-    njs_set_array(retval, array);
+    njs_set_array(&a, array);
 
     for (i = 0; i < nslots; i++) {
-        ret = njs_value_property_i64_set(vm, retval, i, &slots[i].value);
-        if (njs_slow_path(ret == NJS_ERROR)) {
+        ret = njs_value_create_data_prop_i64(vm, &a, i, &slots[i].value, 0);
+        if (njs_slow_path(ret != NJS_OK)) {
             goto exception;
         }
     }
 
     for (; i < length; i++) {
-        ret = njs_value_property_i64_set(vm, retval, i,
-                                      njs_value_arg(&njs_value_undefined));
-        if (njs_slow_path(ret == NJS_ERROR)) {
+        ret = njs_value_create_data_prop_i64(vm, &a, i,
+                                        njs_value_arg(&njs_value_undefined), 0);
+        if (njs_slow_path(ret != NJS_OK)) {
             goto exception;
         }
     }
 
     ret = NJS_OK;
+    njs_set_array(retval, array);
 
 exception:
 
