@@ -17,14 +17,29 @@ struct njs_chb_node_s {
     u_char                  *end;
 };
 
+typedef void *(*njs_chb_alloc_t)(void *pool, size_t size);
+typedef void (*njs_chb_free_t)(void *pool, void *p);
+
 typedef struct {
     njs_bool_t              error;
-    njs_mp_t                *pool;
+
+    void                    *pool;
+    njs_chb_alloc_t         alloc;
+    njs_chb_free_t          free;
+
     njs_chb_node_t          *nodes;
     njs_chb_node_t          *last;
 } njs_chb_t;
 
 
+void njs_chb_init(njs_chb_t *chain, void *pool, njs_chb_alloc_t alloc,
+    njs_chb_free_t free);
+#define NJS_CHB_MP_INIT(chain, vm)                                           \
+    njs_chb_init(chain, njs_vm_memory_pool(vm), (njs_chb_alloc_t) njs_mp_alloc,\
+    (njs_chb_free_t) njs_mp_free)
+#define NJS_CHB_CTX_INIT(chain, ctx)                                         \
+    njs_chb_init(chain, ctx, (njs_chb_alloc_t) js_malloc,                    \
+    (njs_chb_free_t) js_free)
 void njs_chb_append0(njs_chb_t *chain, const char *msg, size_t len);
 void njs_chb_vsprintf(njs_chb_t *chain, size_t size, const char *fmt,
     va_list args);
@@ -48,16 +63,6 @@ void njs_chb_destroy(njs_chb_t *chain);
 #define njs_chb_node_room(n) (size_t) ((n)->end - (n)->pos)
 
 
-njs_inline void
-njs_chb_init(njs_chb_t *chain, njs_mp_t *pool)
-{
-    chain->error = 0;
-    chain->pool = pool;
-    chain->nodes = NULL;
-    chain->last = NULL;
-}
-
-
 njs_inline void
 njs_chb_append_str(njs_chb_t *chain, njs_str_t *str)
 {
@@ -91,6 +96,8 @@ njs_chb_size(njs_chb_t *chain)
 njs_inline int64_t
 njs_chb_utf8_length(njs_chb_t *chain)
 {
+    u_char          *p, *p_end;
+    size_t          size;
     int64_t         len, length;
     njs_chb_node_t  *n;
 
@@ -102,6 +109,23 @@ njs_chb_utf8_length(njs_chb_t *chain)
 
     length = 0;
 
+    while (n != NULL) {
+        p = n->start;
+        size = njs_chb_node_size(n);
+        p_end = p + size;
+
+        while (p < p_end && *p < 0x80) {
+              p++;
+        }
+
+        if (p != p_end) {
+            break;
+        }
+
+        length += size;
+        n = n->next;
+    }
+
     while (n != NULL) {
         len = njs_utf8_length(n->start, njs_chb_node_size(n));
         if (njs_slow_path(len < 0)) {
