@@ -50,7 +50,6 @@ njs_array_iterator_create(njs_vm_t *vm, const njs_value_t *target,
         return NJS_ERROR;
     }
 
-    /* GC retain it->target */
     it->target = *target;
     it->next = 0;
     it->kind = kind;
@@ -163,7 +162,6 @@ njs_array_iterator_next(njs_vm_t *vm, njs_value_t *iterator,
 
 release:
 
-    /* GC release it->target */
     njs_mp_free(vm->mem_pool, it);
     njs_set_invalid(njs_object_value(iterator));
 
@@ -301,7 +299,6 @@ njs_object_iterate(njs_vm_t *vm, njs_iterator_args_t *args,
     njs_value_t         *value, *entry, prop, character;
     const u_char        *p, *end, *pos;
     njs_string_prop_t   string_prop;
-    njs_object_value_t  *object;
 
     value = njs_value_arg(&args->value);
     from = args->from;
@@ -348,15 +345,7 @@ njs_object_iterate(njs_vm_t *vm, njs_iterator_args_t *args,
 
     if (njs_is_string(value) || njs_is_object_string(value)) {
 
-        if (njs_is_string(value)) {
-            object = njs_object_value_alloc(vm, NJS_OBJ_TYPE_STRING, 0, value);
-            if (njs_slow_path(object == NULL)) {
-                return NJS_ERROR;
-            }
-
-            njs_set_object_value(njs_value_arg(&args->value), object);
-        }
-        else {
+        if (!njs_is_string(value)) {
             value = njs_object_value(value);
         }
 
@@ -366,7 +355,7 @@ njs_object_iterate(njs_vm_t *vm, njs_iterator_args_t *args,
         end = p + string_prop.size;
 
         if ((size_t) length == string_prop.size) {
-            /* Byte or ASCII string. */
+            /* ASCII string. */
 
             for (i = from; i < to; i++) {
                 /* This cannot fail. */
@@ -461,7 +450,6 @@ njs_object_iterate_reverse(njs_vm_t *vm, njs_iterator_args_t *args,
     njs_value_t         *entry, *value, prop, character;
     const u_char        *p, *end, *pos;
     njs_string_prop_t   string_prop;
-    njs_object_value_t  *object;
 
     value = njs_value_arg(&args->value);
     from = args->from;
@@ -510,15 +498,7 @@ njs_object_iterate_reverse(njs_vm_t *vm, njs_iterator_args_t *args,
 
     if (njs_is_string(value) || njs_is_object_string(value)) {
 
-        if (njs_is_string(value)) {
-            object = njs_object_value_alloc(vm, NJS_OBJ_TYPE_STRING, 0, value);
-            if (njs_slow_path(object == NULL)) {
-                return NJS_ERROR;
-            }
-
-            njs_set_object_value(njs_value_arg(&args->value), object);
-        }
-        else {
+        if (!njs_is_string(value)) {
             value = njs_object_value(value);
         }
 
@@ -560,7 +540,7 @@ njs_object_iterate_reverse(njs_vm_t *vm, njs_iterator_args_t *args,
             }
 
             while (i-- > to) {
-                pos = njs_utf8_prev(p);
+                pos = njs_utf8_prev(p, string_prop.start);
 
                 /* This cannot fail. */
                 (void) njs_string_new(vm, &character, pos, p - pos , 1);
