@@ -382,7 +382,9 @@ njs_number_is_safe_integer(njs_vm_t *vm, njs_value_t *args, njs_uint_t nargs,
     if (nargs > 1 && njs_is_number(&args[1])) {
         num = njs_number(&args[1]);
 
-        if (num == (int64_t) num && fabs(num) <= NJS_MAX_SAFE_INTEGER) {
+        if (num == njs_unsafe_cast_double_to_int64(num)
+            && fabs(num) <= NJS_MAX_SAFE_INTEGER)
+        {
             integer = 1;
         }
     }
@@ -541,7 +543,7 @@ njs_number_prototype_to_string(njs_vm_t *vm, njs_value_t *args,
         }
 
         if (radix < 2 || radix > 36 || radix != (int) radix) {
-            njs_range_error(vm, NULL);
+            njs_range_error(vm, "radix argument must be between 2 and 36");
             return NJS_ERROR;
         }
 
