@@ -58,61 +58,6 @@ const njs_value_t  njs_string_anonymous =   njs_string("anonymous");
 const njs_value_t  njs_string_memory_error = njs_string("MemoryError");
 
 
-void
-njs_value_retain(njs_value_t *value)
-{
-    njs_string_t  *string;
-
-    if (njs_is_string(value)) {
-
-        if (value->long_string.external != 0xff) {
-            string = value->long_string.data;
-
-            njs_thread_log_debug("retain:%uxD \"%*s\"", string->retain,
-                                 value->long_string.size, string->start);
-
-            if (string->retain != 0xffff) {
-                string->retain++;
-            }
-        }
-    }
-}
-
-
-void
-njs_value_release(njs_vm_t *vm, njs_value_t *value)
-{
-    njs_string_t  *string;
-
-    if (njs_is_string(value)) {
-
-        if (value->long_string.external != 0xff) {
-            string = value->long_string.data;
-
-            njs_thread_log_debug("release:%uxD \"%*s\"", string->retain,
-                                 value->long_string.size, string->start);
-
-            if (string->retain != 0xffff) {
-                string->retain--;
-
-#if 0
-                if (string->retain == 0) {
-                    if ((u_char *) string + sizeof(njs_string_t)
-                        != string->start)
-                    {
-                        njs_memcache_pool_free(vm->mem_pool,
-                                               string->start);
-                    }
-
-                    njs_memcache_pool_free(vm->mem_pool, string);
-                }
-#endif
-            }
-        }
-    }
-}
-
-
 /*
  * A hint value is 0 for numbers and 1 for strings.  The value chooses
  * method calls order specified by ECMAScript 5.1: "valueOf", "toString"
@@ -140,7 +85,6 @@ njs_value_to_primitive(njs_vm_t *vm, njs_value_t *dst, njs_value_t *value,
 
 
     if (njs_is_primitive(value)) {
-        /* GC */
         *dst = *value;
         return NJS_OK;
     }
@@ -192,8 +136,7 @@ njs_value_to_primitive(njs_vm_t *vm, njs_value_t *dst, njs_value_t *value,
 
 
 njs_array_t *
-njs_value_enumerate(njs_vm_t *vm, njs_value_t *value,
-    njs_object_enum_t kind, njs_object_enum_type_t type, njs_bool_t all)
+njs_value_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags)
 {
     njs_int_t           ret;
     njs_value_t         keys;
@@ -201,7 +144,7 @@ njs_value_enumerate(njs_vm_t *vm, njs_value_t *value,
     njs_exotic_slots_t  *slots;
 
     if (njs_is_object(value)) {
-        if (kind == NJS_ENUM_KEYS && (type & NJS_ENUM_STRING)) {
+        if ((flags & NJS_ENUM_KEYS) && (flags & NJS_ENUM_STRING)) {
             slots = njs_object_slots(value);
             if (slots != NULL && slots->keys != NULL) {
                 ret = slots->keys(vm, value, &keys);
@@ -213,7 +156,7 @@ njs_value_enumerate(njs_vm_t *vm, njs_value_t *value,
             }
         }
 
-        return njs_object_enumerate(vm, njs_object(value), kind, type, all);
+        return njs_object_enumerate(vm, njs_object(value), flags);
     }
 
     if (value->type != NJS_STRING) {
@@ -223,13 +166,12 @@ njs_value_enumerate(njs_vm_t *vm, njs_value_t *value,
     obj_val.object = vm->string_object;
     obj_val.value = *value;
 
-    return njs_object_enumerate(vm, (njs_object_t *) &obj_val, kind, type, all);
+    return njs_object_enumerate(vm, (njs_object_t *) &obj_val, flags);
 }
 
 
 njs_array_t *
-njs_value_own_enumerate(njs_vm_t *vm, njs_value_t *value,
-    njs_object_enum_t kind, njs_object_enum_type_t type, njs_bool_t all)
+njs_value_own_enumerate(njs_vm_t *vm, njs_value_t *value, uint32_t flags)
 {
     njs_int_t           ret, len;
     njs_array_t         *values, *entry;
@@ -240,8 +182,7 @@ njs_value_own_enumerate(njs_vm_t *vm, njs_value_t *value,
     if (njs_is_object(value)) {
         slots = njs_object_slots(value);
         if (slots == NULL || slots->keys == NULL) {
-            return njs_object_own_enumerate(vm, njs_object(value), kind, type,
-                                            all);
+            return njs_object_own_enumerate(vm, njs_object(value), flags);
         }
 
         ret = slots->keys(vm, value, &keys);
@@ -249,7 +190,7 @@ njs_value_own_enumerate(njs_vm_t *vm, njs_value_t *value,
             return NULL;
         }
 
-        switch (kind) {
+        switch (njs_object_enum_kind(flags)) {
         case NJS_ENUM_KEYS:
             return njs_array(&keys);
 
@@ -314,8 +255,7 @@ njs_value_own_enumerate(njs_vm_t *vm, njs_value_t *value,
     obj_val.object = vm->string_object;
     obj_val.value = *value;
 
-    return njs_object_own_enumerate(vm, (njs_object_t *) &obj_val, kind,
-                                    type, all);
+    return njs_object_own_enumerate(vm, (njs_object_t *) &obj_val, flags);
 }
 
 
@@ -513,6 +453,13 @@ njs_value_native_function(const njs_value_t *value)
 }
 
 
+void *
+njs_value_ptr(const njs_value_t *value)
+{
+    return njs_data(value);
+}
+
+
 njs_external_ptr_t
 njs_value_external(const njs_value_t *value)
 {
@@ -1622,7 +1569,6 @@ njs_primitive_value_to_string(njs_vm_t *vm, njs_value_t *dst,
         return NJS_ERROR;
 
     case NJS_STRING:
-        /* GC: njs_retain(src); */
         value = src;
         break;
 
