@@ -23,11 +23,7 @@
 
 
 typedef intptr_t                        njs_jump_off_t;
-typedef uint8_t                         njs_vmcode_operation_t;
-
-
-#define NJS_VMCODE_3OPERANDS            0
-#define NJS_VMCODE_2OPERANDS            1
+typedef uint8_t                         njs_vmcode_t;
 
 
 enum {
@@ -92,7 +88,6 @@ enum {
     NJS_VMCODE_LEFT_SHIFT,
     NJS_VMCODE_RIGHT_SHIFT,
     NJS_VMCODE_UNSIGNED_RIGHT_SHIFT,
-    NJS_VMCODE_OBJECT_COPY,
     NJS_VMCODE_TEMPLATE_LITERAL,
     NJS_VMCODE_PROPERTY_IN,
     NJS_VMCODE_PROPERTY_DELETE,
@@ -119,12 +114,6 @@ enum {
 };
 
 
-typedef struct {
-    njs_vmcode_operation_t     operation;
-    uint8_t                    operands;   /* 2 bits */
-} njs_vmcode_t;
-
-
 typedef struct {
     njs_vmcode_t               code;
     njs_index_t                operand1;
@@ -422,7 +411,7 @@ typedef struct {
 } njs_vmcode_await_t;
 
 
-njs_int_t njs_vmcode_interpreter(njs_vm_t *vm, u_char *pc,
+njs_int_t njs_vmcode_interpreter(njs_vm_t *vm, u_char *pc, njs_value_t *retval,
     void *promise_cap, void *async_ctx);
 
 njs_object_t *njs_function_new_object(njs_vm_t *vm, njs_value_t *constructor);
